"""
"""
#Copyright 2009 armsim authors.
#
#This file is part of armsim.
#
#armsim is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#armsim is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with armsim.  If not, see <http://www.gnu.org/licenses/>.

from armsim.interfaces.arminstruction import IARMInstruction
from zope.interface import implements

class ARMMlaInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001111111000000000000011110000
    bitmask  = 0b00000000001000000000000010010000

    def execute(self, proc, inst):
        print "Ejecutaste un mla"

class ARMMulInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001111111000000000000011110000
    bitmask  = 0b00000000000000000000000010010000

    def execute(self, proc, inst):
        print "Ejecutaste un mul"

class ARMSmlaInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001111111100000000000010010000
    bitmask  = 0b00000001000000000000000010000000

    def execute(self, proc, inst):
        print "Ejecutaste un smla<x><y>"

class ARMSmlalInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001111111000000000000011110000
    bitmask  = 0b00000000111000000000000010010000

    def execute(self, proc, inst):
        print "Ejecutaste un smlal"

class ARMSmlalxyInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001111111100000000000010010000
    bitmask  = 0b00000001010000000000000010000000

    def execute(self, proc, inst):
        print "Ejecutaste un smlal<x><y>"

class ARMSmlawyInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001111111100000000000010110000
    bitmask  = 0b00000001001000000000000010000000

    def execute(self, proc, inst):
        print "Ejecutaste un smlaw<y>"

class ARMSmulxyInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001111111100000000000010010000
    bitmask  = 0b00000001011000000000000010000000

    def execute(self, proc, inst):
        print "Ejecutaste un smul<x><y>"

class ARMSmullInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001111111000000000000011110000
    bitmask  = 0b00000000110000000000000010010000

    def execute(self, proc, inst):
        print "Ejecutaste un smull"

class ARMSmulwyInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001111111100000000000010110000
    bitmask  = 0b00000001001000000000000010100000

    def execute(self, proc, inst):
        print "Ejecutaste un smulw<y>"

class ARMUmlalInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001111111000000000000011110000
    bitmask  = 0b00000000101000000000000010010000

    def execute(self, proc, inst):
        print "Ejecutaste un umlal"

class ARMUmullInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001111111000000000000011110000
    bitmask  = 0b00000000100000000000000010010000

    def execute(self, proc, inst):
        print "Ejecutaste un umull"




"""
utils.py

created by Eduardo Diaz

This module defines all the common utils needed in order to
make programming the microcontoller easier
"""
#Copyright 2009 armsim authors.
#
#This file is part of armsim.
#
#armsim is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#armsim is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with armsim.  If not, see <http://www.gnu.org/licenses/>.

from armsim.bitset import Bitset

def getBits(num, bit, num_bits=1, size=32):
    """
    Returns the value of a bit or multiple bits of num.

    Example::
        
        num = 0b10001101
        getBits(num, 0)    # returns 1
        getBits(num, 1)    # returns 0
        getBits(num, 4, 4) # returns 8 (0b1000)
        getBits(num, 0, 8) # returns 141 (0b10001101)

    """
    bs = Bitset(num, size)
    if num_bits == 1:
        return int(bs[bit])
    elif num_bits > 1:
        return bitsetRangeToNum(bs[bit:bit+num_bits])
    return 0

def setBits(num, bit, value, num_bits=1, size=32):
    """
    Sets and returns the value of a bit or multiple bits of num.

    Example::
        
        num = 0b10001101 (141)
        setBits(num, 0, 0)    # returns 140 (0b10001100)
        setBits(num, 1, 1)    # returns 143 (0b10001111)
        setBits(num, 4, 5, 4) # returns 221 (0b11011101)

    """
    bs = Bitset(num, size)
    if num_bits == 1:
        bs[bit] = value
    elif num_bits > 1:
        helper = Bitset(value)
        bs[bit:bit+num_bits] = helper[:]
    return int(bs)

def signValue(num, bits=32):
    """
    Returns signed value of num.

    if msb equals zero, it returns num.

    if msb equals one, it returns negative two's complement of num.
    """

    if getBits(num, bits - 1) == 0:
        return num
    else:
        bs = Bitset(num, bits)
        sc = Bitset(1, bits)
        bs = ~bs
        bs, carry = bs + sc
        return -1 * int(bs)

def signExtend(num, bits, newBits):
    """
    Extends a number from *bits* bits to *newBits* bits

    Example::

        num = 0b0011
        signExtend(num, 4, 10) # returns 0b0000000011
        num = 0b1011
        signExtend(num, 4, 10) # returns 0b1111111011

    """
    bs = Bitset(num, newBits)
    bs[bits:newBits] = bs[bits - 1]

    return int(bs)


def testInstruction(inst, inst_val):
    """
    Function to check if an instruction matches the interface's bitmask


    """
    return (inst_val & inst.testmask) == inst.bitmask

def checkCondition(processor, inst_val):
    """
    Checks if an instruction should be executed according to first 4 bits
    of the instruction
    """
    cond = (inst_val & 0xf0000000) >> 28
    if cond == 0:   #EQ
        return processor.statusFlag('Z') == 1
    elif cond == 1: #NE
        return processor.statusFlag('Z') == 0
    elif cond == 2: #CS/HS
        return processor.statusFlag('C') == 1
    elif cond == 3: #CC/LO
        return processor.statusFlag('C') == 0
    elif cond == 4: #MI
        return processor.statusFlag('N') == 1
    elif cond == 5: #PL
        return processor.statusFlag('N') == 0
    elif cond == 6: #VS
        return processor.statusFlag('V') == 1
    elif cond == 7: #VC
        return processor.statusFlag('V') == 0
    elif cond == 8: #HI
        return (processor.statusFlag('C') == 1) and \
               (processor.statusFlag('Z') == 0)
    elif cond == 9: #LS
        return (processor.statusFlag('C') == 0) or \
               (processor.statusFlag('Z') == 1)
    elif cond == 10: #GE
        return processor.statusFlag('N') == processor.statusFlag('V')
    elif cond == 11: #LT
        return processor.statusFlag('N') != processor.statusFlag('V')
    elif cond == 12: #GT
        return (processor.statusFlag('Z') == 0) and \
               (processor.statusFlag('N') == processor.statusFlag('V'))
    elif cond == 13: #LE
        return (processor.statusFlag('Z') == 1) or \
               (processor.statusFlag('N') != processor.statusFlag('V'))
    elif cond == 14: #AL
        return True


def itoa(x, base=10):
    """
        Converts and integer value to its string representation in base *base*


    """
    isNegative = x < 0
    if isNegative:
        x = -x
    digits = []
    while x > 0:
        x, lastDigit = divmod(x, base)
        digits.append('0123456789abcdefghijklmnopqrstuvwxyz'[lastDigit])
    if isNegative:
        digits.append('-')
    digits.reverse()
    return ''.join(digits)

def bitsetRangeToNum(bitset):
    """
    Returns a bitset range converted to a number

    Usage::

        bitsetRangeToNum(bitset[5:8])

    """
    result = 0
    i = 0
    for x in bitset[:]:
        result |= (int(x) << i)
        i += 1
    return result

def ror(val, n, x):
    """
    Executes a rotate right of *x* bits on number *val*, of *n* bits length

    """

    a = val
    b = a >> x
    c = a << (n - x)
    c = c & 0xffffffff
    d = b | c
    return d

def rol(val, n, x):
    """
    Executes a rotate left of *x* bits on number *val*, of *n* bits length

    """
    a = val
    b = a << x
    b = b & 0xffffffff
    c = a >> (n - x)
    d = b | c
    return d

def asr(val, x, n=32):
    """
    Executes an arithmetic shift right of *x* bits on number *val*, of *n* 
    bits length

    """
    msb = getBits(val, n - 1)
    result = val >> x
    result = setBits(result, n - 1, msb)
    return result

def countSetBits(num):
    """
    Returns number of bits set to one

    Example::
 
        countSetBits(5) #returns 2 (0b0101), 2 bits set to 1
        countSetBits(1) #returns 1 (0b0001), 1 bit set to 1

    """
    set = Bitset(num)
    result = 0
    for n in set:
        if n:
            result += 1
    return result





from zope.component import provideUtility
from armsim.interfaces.arminstruction import IARMInstruction
from armsim.interfaces.arminstruction import IARMDPAddressingMode
from armsim.interfaces.arminstruction import IARMLSAddressingMode
from armsim.interfaces.arminstruction import IARMLSMAddressingMode
import dp
import branch
import mult
import miscellaneous
import status
import ldstr
import semaphore
import exception

#Coprocessor instructions

#Multiply instructions
provideUtility(mult.ARMMlaInstruction(), IARMInstruction, 'mla')
provideUtility(mult.ARMMulInstruction(), IARMInstruction, 'mul')
provideUtility(mult.ARMSmlaInstruction(), IARMInstruction, 'smla')
provideUtility(mult.ARMSmlalInstruction(), IARMInstruction, 'smlal')
provideUtility(mult.ARMSmlalxyInstruction(), IARMInstruction, 'smlal<x><y>')
provideUtility(mult.ARMSmlawyInstruction(), IARMInstruction, 'smlaw<y>')
provideUtility(mult.ARMSmullInstruction(), IARMInstruction, 'smull')
provideUtility(mult.ARMSmulwyInstruction(), IARMInstruction, 'smulw<y>')
provideUtility(mult.ARMSmulxyInstruction(), IARMInstruction, 'smul<x><y>')
provideUtility(mult.ARMUmlalInstruction(), IARMInstruction, 'umlal')
provideUtility(mult.ARMUmullInstruction(), IARMInstruction, 'umull')

#Miscellaneous arithmetic instructions
provideUtility(miscellaneous.ARMClzInstruction(), IARMInstruction, 'clz')

#Semaphore instructions
provideUtility(semaphore.ARMSwpInstruction(), IARMInstruction, 'swp')
provideUtility(semaphore.ARMSwpbInstruction(), IARMInstruction, 'swpb')

#Status register access instructions
provideUtility(status.ARMMrsInstruction(), IARMInstruction, 'mrs')
provideUtility(status.ARMMsrInstruction(), IARMInstruction, 'msr')
provideUtility(status.ARMMsriInstruction(), IARMInstruction, 'msri')

#Branch instructions
provideUtility(branch.ARMBlInstruction(), IARMInstruction, 'bl')
provideUtility(branch.ARMBxInstruction(), IARMInstruction, 'bx')
provideUtility(branch.ARMBlx1Instruction(), IARMInstruction, 'blx1')
provideUtility(branch.ARMBlx2Instruction(), IARMInstruction, 'blx2')

#Load and store instructions
provideUtility(ldstr.ARMLdrtInstruction(), IARMInstruction, 'ldrt')
provideUtility(ldstr.ARMLdrInstruction(), IARMInstruction, 'ldr')
provideUtility(ldstr.ARMLdrbtInstruction(), IARMInstruction, 'ldrbt')
provideUtility(ldstr.ARMLdrbInstruction(), IARMInstruction, 'ldrb')
provideUtility(ldstr.ARMLdrdInstruction(), IARMInstruction, 'ldrd')
provideUtility(ldstr.ARMLdrhInstruction(), IARMInstruction, 'ldrh')
provideUtility(ldstr.ARMLdrsbInstruction(), IARMInstruction, 'ldrsb')
provideUtility(ldstr.ARMLdrshInstruction(), IARMInstruction, 'ldrsh')
provideUtility(ldstr.ARMLdm1Instruction(), IARMInstruction, 'ldm1')
provideUtility(ldstr.ARMLdm2Instruction(), IARMInstruction, 'ldm2')
provideUtility(ldstr.ARMStrtInstruction(), IARMInstruction, 'strt')
provideUtility(ldstr.ARMStrInstruction(), IARMInstruction, 'str')
provideUtility(ldstr.ARMStrbtInstruction(), IARMInstruction, 'strbt')
provideUtility(ldstr.ARMStrbInstruction(), IARMInstruction, 'strb')
provideUtility(ldstr.ARMStrdInstruction(), IARMInstruction, 'strd')
provideUtility(ldstr.ARMStrhInstruction(), IARMInstruction, 'strh')
provideUtility(ldstr.ARMStm1Instruction(), IARMInstruction, 'stm1')
provideUtility(ldstr.ARMStm2Instruction(), IARMInstruction, 'stm2')

#Data processing instructions 16 instructions
provideUtility(dp.ARMAndInstruction(), IARMInstruction, 'and')
provideUtility(dp.ARMEorInstruction(), IARMInstruction, 'eor')
provideUtility(dp.ARMSubInstruction(), IARMInstruction, 'sub')
provideUtility(dp.ARMRsbInstruction(), IARMInstruction, 'rsb')
provideUtility(dp.ARMAddInstruction(), IARMInstruction, 'add')
provideUtility(dp.ARMAdcInstruction(), IARMInstruction, 'adc')
provideUtility(dp.ARMSbcInstruction(), IARMInstruction, 'sbc')
provideUtility(dp.ARMRscInstruction(), IARMInstruction, 'rsc')
provideUtility(dp.ARMTstInstruction(), IARMInstruction, 'tst')
provideUtility(dp.ARMTeqInstruction(), IARMInstruction, 'teq')
provideUtility(dp.ARMCmpInstruction(), IARMInstruction, 'cmp')
provideUtility(dp.ARMCmnInstruction(), IARMInstruction, 'cmn')
provideUtility(dp.ARMOrrInstruction(), IARMInstruction, 'orr')
provideUtility(dp.ARMMovInstruction(), IARMInstruction, 'mov')
provideUtility(dp.ARMBicInstruction(), IARMInstruction, 'bic')
provideUtility(dp.ARMMvnInstruction(), IARMInstruction, 'mvn')

#Exception-generating instructions
provideUtility(exception.ARMBkptInstruction(), IARMInstruction, 'bkpt')
provideUtility(exception.ARMSwiInstruction(), IARMInstruction, 'swi')

#DataProcessing addressing modes
provideUtility(dp.ARMDPAddressingMode1(), IARMDPAddressingMode, 'adp1')
provideUtility(dp.ARMDPAddressingMode2(), IARMDPAddressingMode, 'adp2')
provideUtility(dp.ARMDPAddressingMode3(), IARMDPAddressingMode, 'adp3')
provideUtility(dp.ARMDPAddressingMode4(), IARMDPAddressingMode, 'adp4')
provideUtility(dp.ARMDPAddressingMode5(), IARMDPAddressingMode, 'adp5')
provideUtility(dp.ARMDPAddressingMode6(), IARMDPAddressingMode, 'adp6')
provideUtility(dp.ARMDPAddressingMode7(), IARMDPAddressingMode, 'adp7')
provideUtility(dp.ARMDPAddressingMode8(), IARMDPAddressingMode, 'adp8')
provideUtility(dp.ARMDPAddressingMode9(), IARMDPAddressingMode, 'adp9')
provideUtility(dp.ARMDPAddressingMode10(), IARMDPAddressingMode, 'adp10')

#Load Store word or byte addressing modes
provideUtility(ldstr.ARMLSAddressingMode1(), IARMLSAddressingMode, 'als1')
provideUtility(ldstr.ARMLSAddressingMode2(), IARMLSAddressingMode, 'als2')
provideUtility(ldstr.ARMLSAddressingMode3(), IARMLSAddressingMode, 'als3')
provideUtility(ldstr.ARMLSAddressingMode4(), IARMLSAddressingMode, 'als4')
provideUtility(ldstr.ARMLSAddressingMode5(), IARMLSAddressingMode, 'als5')
provideUtility(ldstr.ARMLSAddressingMode6(), IARMLSAddressingMode, 'als6')

#Load Store Multiple word or byte addressing modes
provideUtility(ldstr.ARMLSMAddressingMode1(), IARMLSMAddressingMode, 'alsm1')
provideUtility(ldstr.ARMLSMAddressingMode2(), IARMLSMAddressingMode, 'alsm2')
provideUtility(ldstr.ARMLSMAddressingMode3(), IARMLSMAddressingMode, 'alsm3')
provideUtility(ldstr.ARMLSMAddressingMode4(), IARMLSMAddressingMode, 'alsm4')



"""
"""
#Copyright 2009 armsim authors.
#
#This file is part of armsim.
#
#armsim is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#armsim is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with armsim.  If not, see <http://www.gnu.org/licenses/>.

from zope.interface import Interface
from zope.interface import Attribute

class IARMProcessor(Interface):
    """This interface defines the ARM processor"""

    memory = Attribute("Processor memory buffer")

    r0  = Attribute("R0 register")
    r1  = Attribute("R1 register")
    r2  = Attribute("R2 register")
    r3  = Attribute("R3 register")
    r4  = Attribute("R4 register")
    r5  = Attribute("R5 register")
    r6  = Attribute("R6 register")
    r7  = Attribute("R7 register")
    r8  = Attribute("R8 register")
    r9  = Attribute("R9 register")
    r10 = Attribute("R10 register")
    r11 = Attribute("R11 register")
    r12 = Attribute("R12 register")
    r13 = Attribute("R13 register")
    r14 = Attribute("R14 register, holds SP")
    r15 = Attribute("R15 register, holds PC")

    #Supervisor mode registers
    r13_svc = Attribute("Supervisor mode R13 register")
    r14_svc = Attribute("Supervisor mode R14 register")
    spsr_svc = Attribute("Supervisor mode SPSR status register")

    #Abort mode registers
    r13_abt = Attribute("Abort mode R13 register")
    r14_abt = Attribute("Abort mode R14 register")
    spsr_abt = Attribute("Abort mode SPSR status register")

    #Undefined mode registers
    r13_und = Attribute("Undefined mode R13 register")
    r14_und = Attribute("Undefined mode R14 register")
    spsr_und = Attribute("Undefined mode SPSR status register")

    #IRQ mode registers
    r13_irq = Attribute("IRQ mode R13 register")
    r14_irq = Attribute("IRQ mode R14 register")
    spsr_irq = Attribute("IRQ mode SPSR status register")

    #FIQ mode registers
    r8_fiq = Attribute("FIQ mode R8 register")
    r9_fiq = Attribute("FIQ mode R9 register")
    r10_fiq = Attribute("FIQ mode R10 register")
    r11_fiq = Attribute("FIQ mode R11 register")
    r12_fiq = Attribute("FIQ mode R12 register")
    r13_fiq = Attribute("FIQ mode R13 register")
    r14_fiq = Attribute("FIQ mode R14 register")
    spsr_fiq = Attribute("FIQ mode SPSR status register")

    cpsr = Attribute("CPSR Status register")

    def halt():
        """Pauses execution"""

    def printStatus():
        """Prints execution state, current PC and value of all registers"""

    def initMemory(size):
        """Initializes the memory to the size of the parameter passed"""

    def reset():
        """Resets the processor, CAUTION registers are not erased"""

    def fetch():
        """Looks for the next instruction and returns it"""

    def resume():
        """
           Resumes execution at current PC, you can use this method along with
           halt to debug, and change register values at runtime
           Example: 
                processor.halt()

                processor.r0
                45L
                processor.r0 = 15
                processor.resume()
        """

    def step():
        """
           Jumps to the next instruction, processor automatically executes this
           after a fetch
        """

    def setStatusFlag(flag, value):
        """
           Used to change status flags, values allowed to the flag variable are:
           'M' for mode
           'T' for thumb instruction set (altough this isn't supported)
           'J' for Jazzelle instruction set (it isn't supported yet)

           You can read about the rest of the bits
           in the ARM Architecture Manual
           'F' 
           'I'
           'A'
           'E'
           'GE'
           'Q' 

           and condition code flags:
           'V' oVerflow
           'C' Carry 
           'Z' Zero
           'N' Negative
        """

    def statusFlag(flag):
        """
           Returns the value of a status flag, 
           values allowed to the flag variable are:
           'M' for mode
           'T' for thumb instruction set (altough this isn't supported)
           'J' for Jazzelle instruction set (it isn't supported yet)

           You can read about the rest of the bits
           in the ARM Architecture Manual
           'F' 
           'I'
           'A'
           'E'
           'GE'
           'Q' 

           and condition code flags:
           'V' oVerflow
           'C' Carry 
           'Z' Zero
           'N' Negative
        """

    def mode():
        """Shortcut to statusFlag('M')"""



"""
bitset.py

Written by Geremy Condra

Licensed under GPLv3

Released 3 May 2009

This module provides a simple bitset implementation
for Python.

Modified by Eduardo Diaz
Mayo 8 2009

"""
#Copyright 2009 armsim authors.
#
#This file is part of armsim.
#
#armsim is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#armsim is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with armsim.  If not, see <http://www.gnu.org/licenses/>

from collections import Sequence
import math

class Bitset(Sequence):
	"""A very simple bitset implementation for Python.

	Note that the rightmost
	index is the LSB, and that
	is 0.

	Usage:
		>>> b = Bitset(5)
		>>> b
		Bitset(101)
		>>> b[:]
		[True, False, True]
		>>> b[0] = False
		>>> b
		Bitset(001)
		>>> b << 1
		Bitset(010)
		>>> b >> 1
		Bitset(000)
		>>> b & 1
		Bitset(001)
		>>> b | 2
		Bitset(011)
		>>> b ^ 6
		Bitset(111)
		>>> ~b
		Bitset(110)
	"""

	value = 0
	length = 0

	@classmethod
	def from_sequence(cls, seq):
		"""Iterates over the sequence to produce a new Bitset.

		As in integers, the 0 position represents the LSB.
		"""
		n = 0
		for index, value in enumerate(reversed(seq)):
			n += 2**index * bool(int(value))
		b = Bitset(n)
		return b

	def __init__(self, value=0, length=0):
		"""Creates a Bitset with the given integer value."""
		self.value = value
		try: self.length = length or math.floor(math.log(value, 2)) + 1
		except Exception: self.length = 0

	def __and__(self, other):
		b = Bitset(self.value & int(other))
		b.length = max((self.length, b.length))
		return b

	def __or__(self, other):
		b = Bitset(self.value | int(other))
		b.length = max((self.length, b.length))
		return b

	def __invert__(self):
		#b = Bitset(~self.value)
		#b.length = max((self.length, b.length))
		b = Bitset(self.value, self.length)
		i = 0
		while i < self.length:
			b[i] = not b[i]
			i += 1
		return b

	def __xor__(self, value):
		b = Bitset(self.value ^ int(value))
		b.length = max((self.length, b.length))
		return b

	def __lshift__(self, value):
		b = Bitset(self.value << int(value))
		b.length = max((self.length, b.length))
		return b

	def __rshift__(self, value):
		b = Bitset(self.value >> int(value))
		b.length = max((self.length, b.length))
		return b

	def __eq__(self, other):
		try:
			return self.value == other.value
		except Exception:
			return self.value == other

	def __int__(self):
		return self.value

	def __str__(self):
		s = ""
		for i in self[::-1]:
			s += "1" if i else "0"
		return s

	def __repr__(self):
		return "Bitset(%s)" % str(self)

	def __getitem__(self, s):
		"""Gets the specified position.

		0 represents the LSB.
		"""
		try:
			start, stop, step = s.indices(len(self))
			results = []
			for position in range(start, stop, step):
				#pos = len(self) - position - 1
                                pos = position
				results.append(bool(self.value & (1 << pos)))
			return results
		except:
			#pos = len(self) - s - 1
                        pos = s
			return bool(self.value & (1 << pos))
 
	def __setitem__(self, s, value):
		"""Sets the specified position/s to value.

		0 represents the LSB.
		"""
		try:
			start, stop, step = s.indices(len(self))
			for position in range(start, stop, step):
				#pos = len(self) - position - 1
                                pos = position
				if value: self.value |= (1 << pos)
				else: self.value &= ~(1 << pos)
			maximum_position = max((start + 1, stop, len(self)))
			self.length = maximum_position
		except:
			#pos = len(self) - s - 1
                        pos = s
			if value: self.value |= (1 << pos)
			else: self.value &= ~(1 << pos)
			if len(self) < pos: self.length = pos
		return self

	def __add__(self, other):
		carry  = 0
                size   = int(max((other.length, self.length)))
                result = Bitset(0, size)
                for i in range(size):
			num = int(other[i]) + int(self[i]) + carry
			if num >= 2:
				carry = 1
				num -= 2
			else:
				carry = 0
			result[i] = num
			
		return (result, carry)
 
	def __sub__(self, other):
		carry  = 0
                size   = int(max((other.length, self.length)))
                result = Bitset(0, size)
                for i in range(size):
                        substract = int(other[i]) + carry
			num = int(self[i]) - substract
			if num == -2:
				carry = 1
				num = 0
			elif num == -1:
				carry = 1
				num = 1
			else:
				carry = 0
				
			result[i] = num
			
		return (result, carry)

	def __iter__(self):
		"""Iterates over the values in the bitset."""
		for i in self[:]:
			yield i

	def __len__(self):
		"""Returns the length of the bitset."""
		return self.length



"""
"""
#Copyright 2009 armsim authors.
#
#This file is part of armsim.
#
#armsim is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#armsim is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with armsim.  If not, see <http://www.gnu.org/licenses/>.

from armsim.interfaces.armprocessor import IARMProcessor
from armsim.interfaces.arminstruction import IARMInstruction
from armsim.utils import testInstruction
from armsim.bitset import Bitset
from armsim import utils
from zope.interface import implements
from zope import component
from armsim.processor import ProcessorMode
from array import array

class ARM9Processor(object):
    implements(IARMProcessor)

    memory = None
    __running = True
 
    def __init__(self):
        regs = {}

        for name in IARMProcessor.names():
            if name != 'memory':
                regs[name] = 0

        self.__dict__['regs'] = regs
        self.reset()
  
    def halt(self):
        self.__running = False

    def printStatus(self):
        regs = self.regs
        if self.__running:
            print "Processor running at 0x%x" % self.pc
        else:
            print "Processor halted execution at 0x%x" % self.pc
        for k, v in regs.items():
            print "%s=%s" % (k, v)
   
    def initMemory(self, size):
        self.memory = array('B', [0 for x in range(size)])
        
    def reset(self):
        self.regs['r14_svc'] = 0
        self.regs['spsr_svc'] = 0
        self.regs['cpsr'] = ProcessorMode.Supervisor
        self.regs['cpsr'] |= 0b11000000

    def fetch(self):
        inst = 0
        nextAddr = self.pc
        for i in range(4):
            inst |= self.memory[nextAddr + i] << (i * 8)

        return inst

    def resume(self):
        self.__running = True
        self.run()

    def run(self):
        processor = self
        instructions = component.getAllUtilitiesRegisteredFor(IARMInstruction)

        while self.__running:
            inst = processor.fetch()
            processor.step()
            executed = False
            if inst == 0:
                break
            for instruction in instructions:
                if testInstruction(instruction, inst):
                    instruction.execute(processor, inst)
                    executed = True
                    break

            if not executed:
                print "Instruction not recognized ", utils.itoa(inst, 2).zfill(32)
        self.__running = False


    def step(self):
        self.pc += 4

    def setStatusFlag(self, flag, value):
        cpsr = Bitset(self.cpsr, 32)
        if flag == 'M':
            helper = Bitset(value)
            cpsr[0:5] = helper[:]
        elif flag == 'T':
            cpsr[5] = value
        elif flag == 'F':
            cpsr[6] = value
        elif flag == 'I':
            cpsr[7] = value
        elif flag == 'A':
            cpsr[8] = value
        elif flag == 'E':
            cpsr[9] = value
        elif flag == 'GE':
            helper = Bitset(value)
            cpsr[16:20] = helper[:]
        elif flag == 'J':
            cpsr[24] = value
        elif flag == 'Q':
            cpsr[27] = value
        elif flag == 'V':
            cpsr[28] = value
        elif flag == 'C':
            cpsr[29] = value
        elif flag == 'Z':
            cpsr[30] = value
        elif flag == 'N':
            cpsr[31] = value
        self.cpsr = int(cpsr)

    def statusFlag(self, flag):
        cpsr = Bitset(self.cpsr, 32)
        if flag == 'M':
            return utils.bitsetRangeToNum(cpsr[0:5])
        elif flag == 'T':
            return int(cpsr[5])
        elif flag == 'F':
            return int(cpsr[6])
        elif flag == 'I':
            return int(cpsr[7])
        elif flag == 'A':
            return int(cpsr[8])
        elif flag == 'E':
            return int(cpsr[9])
        elif flag == 'GE':
            return utils.bitsetRangeToNum(cpsr[16:20])
        elif flag == 'J':
            return int(cpsr[24])
        elif flag == 'Q':
            return int(cpsr[27])
        elif flag == 'V':
            return int(cpsr[28])
        elif flag == 'C':
            return int(cpsr[29])
        elif flag == 'Z':
            return int(cpsr[30])
        elif flag == 'N':
            return int(cpsr[31])

    def mode(self):
        return self.regs['cpsr'] & 0x0000001F;

    def __setattr__(self, name, value):
        regs = self.__dict__['regs']

        if name == 'pc':
            self.__setattr__('r15', value)
        if name == 'lr':
            self.__setattr__('r14', value)
        if name == 'sp':
            self.__setattr__('r13', value)
    
        if self.mode() == ProcessorMode.Supervisor:
            nname = name + '_svc'
            if nname in regs:
                regs[nname] = value
    
        if self.mode() == ProcessorMode.Abort:
            nname = name + '_abt'
            if nname in regs:
                regs[nname] = value
            
        if self.mode() == ProcessorMode.Undefined:
            nname = name + '_und'
            if nname in regs:
                regs[nname] = value
    
        if self.mode() == ProcessorMode.IRQ:
            nname = name + '_irq'
            if nname in regs:
                regs[nname] = value
    
        if self.mode() == ProcessorMode.FIQ:
            nname = name + '_fiq'
            if nname in regs:
                regs[nname] = value
    
        if name in regs:
            regs[name] = value
        
        object.__setattr__(self, name, value)

    def __getattr__(self, name):
        if name == 'pc':
            return self.__getattr__('r15')
        if name == 'lr':
            return self.__getattr__('r14')
        if name == 'sp':
            return self.__getattr__('r13')
        if name == '__members__':
            return self.regs.keys() + ['pc', 'lr', 'sp']
        if self.mode() == ProcessorMode.Supervisor:
            nname = name + '_svc'
            if nname in self.regs:
                return self.regs[nname]

        if self.mode() == ProcessorMode.Abort:
            nname = name + '_abt'
            if nname in self.regs:
                return self.regs[nname]
            
        if self.mode() == ProcessorMode.Undefined:
            nname = name + '_und'
            if nname in self.regs:
                return self.regs[nname]

        if self.mode() == ProcessorMode.IRQ:
            nname = name + '_irq'
            if nname in self.regs:
                return self.regs[nname]

        if self.mode() == ProcessorMode.FIQ:
            nname = name + '_fiq'
            if nname in self.regs:
                return self.regs[nname]

        if name in self.regs:
            return self.regs[name]
        
        #raise AttributeException('no such attribute ' + name)
        return self.__dict__[name]

    def saveAddr(self, address, value, num_bytes=4, endianess='L'):
        nextAddr = address

        if nextAddr < 0:
            raise Exception("Address out of range")

        if (nextAddr + num_bytes) > len(self.memory):
            raise Exception("Address out of range")

        if(endianess == 'L'):
            for i in range(num_bytes):
                self.memory[nextAddr + i] = utils.getBits(value, i * 8, 8)
        else:
            for i in range(num_bytes):
                j = num_bytes - 1 - i
                self.memory[nextAddr + i] = utils.getBits(value, j * 8, 8)

    
    def readAddr(self, address, num_bytes=4, endianess='L'):
        location = 0
        nextAddr = address

        if nextAddr < 0:
            raise Exception("Address out of range")

        if (nextAddr + num_bytes) > len(self.memory):
            raise Exception("Address out of range")

        if(endianess == 'L'):
            for i in range(num_bytes):
                location |= self.memory[nextAddr + i] << (i * 8)
        else:
            for i in range(num_bytes):
                j = num_bytes - 1 - i
                location |= self.memory[nextAddr + i] << (j * 8)

        return location
        




"""
Data Processing instructions module

Here is where you can find the data processing instructions which are

* and
* eor
* sub

"""
#Copyright 2009 armsim authors.
#
#This file is part of armsim.
#
#armsim is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#armsim is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with armsim.  If not, see <http://www.gnu.org/licenses/>.


from armsim.interfaces.arminstruction import IARMInstruction
from armsim.interfaces.arminstruction import IARMDPAddressingMode
from armsim.utils import checkCondition
from armsim.utils import testInstruction
from armsim.bitset import Bitset
from armsim import utils
from zope.interface import implements
from zope import component

class ARMAndInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001101111000000000000000000000
    bitmask  = 0b00000000000000000000000000000000

    def execute(self, proc, inst):
        shifter_operand   = 0
        shifter_carry_out = 0
        if not checkCondition(proc, inst):
            return

        S  = utils.getBits(inst, 20)
        I  = utils.getBits(inst, 25)
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)
        rn = utils.getBits(inst, 16, 4)
        rn = 'r' + str(rn)

        modes =  component.getAllUtilitiesRegisteredFor(IARMDPAddressingMode)
        for mode in modes:
            if testInstruction(mode, inst):
                shifter_operand, shifter_carry_out = mode.getVal(proc, inst)
                break

        setattr(proc, rd, getattr(proc, rn) & shifter_operand)
        if S == 1 and rd == 'r15':
            try:
                proc.cpsr = proc.spsr
            except:
                print "UNPREDICTABLE"
        elif S == 1:
            proc.setStatusFlag('N', utils.getBits(getattr(proc, rd), 31))
            proc.setStatusFlag('Z', int(getattr(proc, rd) == 0))
            proc.setStatusFlag('C', shifter_carry_out)

class ARMEorInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001101111000000000000000000000
    bitmask  = 0b00000000001000000000000000000000

    def execute(self, proc, inst):
        shifter_operand   = 0
        shifter_carry_out = 0
        if not checkCondition(proc, inst):
            return

        S  = utils.getBits(inst, 20)
        I  = utils.getBits(inst, 25)
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)
        rn = utils.getBits(inst, 16, 4)
        rn = 'r' + str(rn)
        rdVal = getattr(proc, rd)
        rnVal = getattr(proc, rn)

        modes =  component.getAllUtilitiesRegisteredFor(IARMDPAddressingMode)
        for mode in modes:
            if testInstruction(mode, inst):
                shifter_operand, shifter_carry_out = mode.getVal(proc, inst)
                break

        setattr(proc, rd, rnVal ^ shifter_operand)
        if S == 1 and rd == 'r15':
            try:
                proc.cpsr = proc.spsr
            except:
                print "UNPREDICTABLE"
        elif S == 1:
            proc.setStatusFlag('N', utils.getBits(getattr(proc, rd), 31))
            proc.setStatusFlag('Z', int(getattr(proc, rd) == 0))
            proc.setStatusFlag('C', shifter_carry_out)
        


class ARMSubInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001101111000000000000000000000
    bitmask  = 0b00000000010000000000000000000000

    def execute(self, proc, inst):
        shifter_operand   = 0
        shifter_carry_out = 0
        if not checkCondition(proc, inst):
            return

        S  = utils.getBits(inst, 20)
        I  = utils.getBits(inst, 25)
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)
        rn = utils.getBits(inst, 16, 4)
        rn = 'r' + str(rn)
        rdVal = getattr(proc, rd)
        rnVal = getattr(proc, rn)
        rnBitset = Bitset(rnVal, 32)

        modes =  component.getAllUtilitiesRegisteredFor(IARMDPAddressingMode)
        for mode in modes:
            if testInstruction(mode, inst):
                shifter_operand, shifter_carry_out = mode.getVal(proc, inst)
                break
        
        shifter_bitset = Bitset(shifter_operand, 32)

        result, borrow = rnBitset - shifter_bitset
        setattr(proc, rd, int(result))

        if S == 1 and rd == 'r15':
            try:
                proc.cpsr = proc.spsr
            except:
                print "UNPREDICTABLE"
        elif S == 1:
            if utils.getBits(rdVal, 31) == 0 and utils.getBits(int(result), 31) == 1:
                v = 1
            else:
                v = 0
            proc.setStatusFlag('N', utils.getBits(getattr(proc, rd), 31))
            proc.setStatusFlag('Z', int(getattr(proc, rd) == 0))
            proc.setStatusFlag('C', int(borrow == 0))
            proc.setStatusFlag('V', v)

class ARMRsbInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001101111000000000000000000000
    bitmask  = 0b00000000011000000000000000000000

    def execute(self, proc, inst):
        shifter_operand   = 0
        shifter_carry_out = 0
        if not checkCondition(proc, inst):
            return

        S  = utils.getBits(inst, 20)
        I  = utils.getBits(inst, 25)
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)
        rn = utils.getBits(inst, 16, 4)
        rn = 'r' + str(rn)
        rdVal = getattr(proc, rd)
        rnVal = getattr(proc, rn)
        rnBitset = Bitset(rnVal, 32)

        modes =  component.getAllUtilitiesRegisteredFor(IARMDPAddressingMode)
        for mode in modes:
            if testInstruction(mode, inst):
                shifter_operand, shifter_carry_out = mode.getVal(proc, inst)
                break
        
        shifter_bitset = Bitset(shifter_operand, 32)

        result, borrow = rshifter_bitset - rnBitset
        setattr(proc, rd, int(result))

        if S == 1 and rd == 'r15':
            try:
                proc.cpsr = proc.spsr
            except:
                print "UNPREDICTABLE"
        elif S == 1:
            if utils.getBits(shifter_operand, 31) == 0 and utils.getBits(int(result), 31) == 1:
                v = 1
            else:
                v = 0
            proc.setStatusFlag('N', utils.getBits(getattr(proc, rd), 31))
            proc.setStatusFlag('Z', int(getattr(proc, rd) == 0))
            proc.setStatusFlag('C', int(borrow == 0))
            proc.setStatusFlag('V', v)

class ARMAddInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001101111000000000000000000000
    bitmask  = 0b00000000100000000000000000000000

    def execute(self, proc, inst):
        shifter_operand   = 0
        shifter_carry_out = 0
        if not checkCondition(proc, inst):
            return

        S  = utils.getBits(inst, 20)
        I  = utils.getBits(inst, 25)
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)
        rn = utils.getBits(inst, 16, 4)
        rn = 'r' + str(rn)
        rdVal = getattr(proc, rd)
        rnVal = getattr(proc, rn)
        rnBitset = Bitset(rnVal, 32)

        modes =  component.getAllUtilitiesRegisteredFor(IARMDPAddressingMode)
        for mode in modes:
            if testInstruction(mode, inst):
                shifter_operand, shifter_carry_out = mode.getVal(proc, inst)
                break
        
        shifter_bitset = Bitset(shifter_operand, 32)

        result, carry = rnBitset + shifter_bitset
        setattr(proc, rd, int(result))

        if S == 1 and rd == 'r15':
            try:
                proc.cpsr = proc.spsr
            except:
                print "UNPREDICTABLE"
        elif S == 1:
            if utils.getBits(rdVal, 31) == 0 and utils.getBits(int(result), 31) == 1:
                v = 1
            else:
                v = 0
            proc.setStatusFlag('N', utils.getBits(getattr(proc, rd), 31))
            proc.setStatusFlag('Z', int(getattr(proc, rd) == 0))
            proc.setStatusFlag('C', carry)
            proc.setStatusFlag('V', v)

class ARMAdcInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001101111000000000000000000000
    bitmask  = 0b00000000101000000000000000000000

    def execute(self, proc, inst):
        shifter_operand   = 0
        shifter_carry_out = 0
        if not checkCondition(proc, inst):
            return

        S  = utils.getBits(inst, 20)
        I  = utils.getBits(inst, 25)
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)
        rn = utils.getBits(inst, 16, 4)
        rn = 'r' + str(rn)
        rdVal = getattr(proc, rd)
        rnVal = getattr(proc, rn)
        rnBitset = Bitset(rnVal, 32)
        carryBitset = Bitset(proc.statusFlag('C'), 32)

        modes =  component.getAllUtilitiesRegisteredFor(IARMDPAddressingMode)
        for mode in modes:
            if testInstruction(mode, inst):
                shifter_operand, shifter_carry_out = mode.getVal(proc, inst)
                break
        
        shifter_bitset = Bitset(shifter_operand, 32)

        result, carry = rnBitset + shifter_bitset
        resultBitset  = Bitset(result, 32)
        result, carry = resultBitset + carryBitset
        setattr(proc, rd, int(result))

        if S == 1 and rd == 'r15':
            try:
                proc.cpsr = proc.spsr
            except:
                print "UNPREDICTABLE"
        elif S == 1:
            if utils.getBits(rdVal, 31) == 0 and utils.getBits(int(result), 31) == 1:
                v = 1
            else:
                v = 0
            proc.setStatusFlag('N', utils.getBits(getattr(proc, rd), 31))
            proc.setStatusFlag('Z', int(getattr(proc, rd) == 0))
            proc.setStatusFlag('C', carry)
            proc.setStatusFlag('V', v)

class ARMSbcInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001101111000000000000000000000
    bitmask  = 0b00000000110000000000000000000000

    def execute(self, proc, inst):
        shifter_operand   = 0
        shifter_carry_out = 0
        if not checkCondition(proc, inst):
            return

        S  = utils.getBits(inst, 20)
        I  = utils.getBits(inst, 25)
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)
        rn = utils.getBits(inst, 16, 4)
        rn = 'r' + str(rn)
        rdVal = getattr(proc, rd)
        rnVal = getattr(proc, rn)
        rnBitset = Bitset(rnVal, 32)
        carryBitset = Bitset(proc.statusFlag('C'), 32)

        modes =  component.getAllUtilitiesRegisteredFor(IARMDPAddressingMode)
        for mode in modes:
            if testInstruction(mode, inst):
                shifter_operand, shifter_carry_out = mode.getVal(proc, inst)
                break
        
        shifter_bitset = Bitset(shifter_operand, 32)

        result, carry = rnBitset - shifter_bitset
        resultBitset  = Bitset(result, 32)
        carryBitset[0] = not carryBitset[0]
        result, carry = resultBitset - carryBitset
        setattr(proc, rd, int(result))

        if S == 1 and rd == 'r15':
            try:
                proc.cpsr = proc.spsr
            except:
                print "UNPREDICTABLE"
        elif S == 1:
            if utils.getBits(rdVal, 31) == 0 and utils.getBits(int(result), 31) == 1:
                v = 1
            else:
                v = 0
            proc.setStatusFlag('N', utils.getBits(getattr(proc, rd), 31))
            proc.setStatusFlag('Z', int(getattr(proc, rd) == 0))
            proc.setStatusFlag('C', int(carry == 0))
            proc.setStatusFlag('V', v)

class ARMRscInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001101111000000000000000000000
    bitmask  = 0b00000000111000000000000000000000

    def execute(self, proc, inst):
        shifter_operand   = 0
        shifter_carry_out = 0
        if not checkCondition(proc, inst):
            return

        S  = utils.getBits(inst, 20)
        I  = utils.getBits(inst, 25)
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)
        rn = utils.getBits(inst, 16, 4)
        rn = 'r' + str(rn)
        rdVal = getattr(proc, rd)
        rnVal = getattr(proc, rn)
        rnBitset = Bitset(rnVal, 32)
        carryBitset = Bitset(proc.statusFlag('C'), 32)

        modes =  component.getAllUtilitiesRegisteredFor(IARMDPAddressingMode)
        for mode in modes:
            if testInstruction(mode, inst):
                shifter_operand, shifter_carry_out = mode.getVal(proc, inst)
                break
        
        shifter_bitset = Bitset(shifter_operand, 32)

        result, carry = shifter_bitset - rnBitset
        resultBitset  = Bitset(result, 32)
        carryBitset[0] = not carryBitset[0]
        result, carry = resultBitset - carryBitset
        setattr(proc, rd, int(result))

        if S == 1 and rd == 'r15':
            try:
                proc.cpsr = proc.spsr
            except:
                print "UNPREDICTABLE"
        elif S == 1:
            if utils.getBits(rdVal, 31) == 0 and utils.getBits(int(result), 31) == 1:
                v = 1
            else:
                v = 0
            proc.setStatusFlag('N', utils.getBits(getattr(proc, rd), 31))
            proc.setStatusFlag('Z', int(getattr(proc, rd) == 0))
            proc.setStatusFlag('C', int(carry == 0))
            proc.setStatusFlag('V', v)
        print "Ejecutaste un rsc"

class ARMTstInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001101111100000000000000000000
    bitmask  = 0b00000001000100000000000000000000

    def execute(self, proc, inst):
        shifter_operand   = 0
        shifter_carry_out = 0
        if not checkCondition(proc, inst):
            return

        rn = utils.getBits(inst, 16, 4)
        rn = 'r' + str(rn)
        rnVal = getattr(proc, rn)

        modes =  component.getAllUtilitiesRegisteredFor(IARMDPAddressingMode)
        for mode in modes:
            if testInstruction(mode, inst):
                shifter_operand, shifter_carry_out = mode.getVal(proc, inst)
                break

        alu_out = rnVal & shifter_operand

        
        proc.setStatusFlag('N', utils.getBits(alu_out, 31))
        proc.setStatusFlag('Z', int(alu_out == 0))
        proc.setStatusFlag('C', shifter_carry_out)

class ARMTeqInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001101111100000000000000000000
    bitmask  = 0b00000001001100000000000000000000

    def execute(self, proc, inst):
        shifter_operand   = 0
        shifter_carry_out = 0
        if not checkCondition(proc, inst):
            return

        rn = utils.getBits(inst, 16, 4)
        rn = 'r' + str(rn)
        rnVal = getattr(proc, rn)

        modes =  component.getAllUtilitiesRegisteredFor(IARMDPAddressingMode)
        for mode in modes:
            if testInstruction(mode, inst):
                shifter_operand, shifter_carry_out = mode.getVal(proc, inst)
                break

        alu_out = rnVal ^ shifter_operand
        
        proc.setStatusFlag('N', utils.getBits(alu_out, 31))
        proc.setStatusFlag('Z', int(alu_out == 0))
        proc.setStatusFlag('C', shifter_carry_out)

class ARMCmpInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001101111100000000000000000000
    bitmask  = 0b00000001010100000000000000000000

    def execute(self, proc, inst):
        shifter_operand   = 0
        shifter_carry_out = 0
        if not checkCondition(proc, inst):
            return

        rn = utils.getBits(inst, 16, 4)
        rn = 'r' + str(rn)
        rnVal = getattr(proc, rn)

        modes =  component.getAllUtilitiesRegisteredFor(IARMDPAddressingMode)
        for mode in modes:
            if testInstruction(mode, inst):
                shifter_operand, shifter_carry_out = mode.getVal(proc, inst)
                break

        rnBitset      = Bitset(rnVal, 32)
        shifterBitset = Bitset(shifter_operand, 32)
        alu_out, borrow = rnBitset - shifterBitset
        
        if utils.getBits(rnVal, 31) == 0 and utils.getBits(int(alu_out), 31) == 1:
            v = 1
        else:
            v = 0
        proc.setStatusFlag('N', utils.getBits(int(alu_out), 31))
        proc.setStatusFlag('Z', int(int(alu_out) == 0))
        proc.setStatusFlag('C', int(borrow == 0))
        proc.setStatusFlag('V', v)

class ARMCmnInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001101111100000000000000000000
    bitmask  = 0b00000001011100000000000000000000

    def execute(self, proc, inst):
        shifter_operand   = 0
        shifter_carry_out = 0
        if not checkCondition(proc, inst):
            return

        rn = utils.getBits(inst, 16, 4)
        rn = 'r' + str(rn)
        rnVal = getattr(proc, rn)

        modes =  component.getAllUtilitiesRegisteredFor(IARMDPAddressingMode)
        for mode in modes:
            if testInstruction(mode, inst):
                shifter_operand, shifter_carry_out = mode.getVal(proc, inst)
                break

        rnBitset      = Bitset(rnVal, 32)
        shifterBitset = Bitset(shifter_operand, 32)
        alu_out, carry = rnBitset + shifter_operand
        
        if utils.getBits(rdVal, 31) == 0 and utils.getBits(int(result), 31) == 1:
            v = 1
        else:
            v = 0
        proc.setStatusFlag('N', utils.getBits(int(alu_out), 31))
        proc.setStatusFlag('Z', int(int(alu_out) == 0))
        proc.setStatusFlag('C', carry)
        proc.setStatusFlag('V', v)

class ARMOrrInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001101111000000000000000000000
    bitmask  = 0b00000001100000000000000000000000

    def execute(self, proc, inst):
        shifter_operand   = 0
        shifter_carry_out = 0
        if not checkCondition(proc, inst):
            return

        S  = utils.getBits(inst, 20)
        I  = utils.getBits(inst, 25)
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)
        rn = utils.getBits(inst, 16, 4)
        rn = 'r' + str(rn)

        modes =  component.getAllUtilitiesRegisteredFor(IARMDPAddressingMode)
        for mode in modes:
            if testInstruction(mode, inst):
                shifter_operand, shifter_carry_out = mode.getVal(proc, inst)
                break

        setattr(proc, rd, getattr(proc, rn) | shifter_operand)
        if S == 1 and rd == 'r15':
            try:
                proc.cpsr = proc.spsr
            except:
                print "UNPREDICTABLE"
        elif S == 1:
            proc.setStatusFlag('N', utils.getBits(getattr(proc, rd), 31))
            proc.setStatusFlag('Z', int(getattr(proc, rd) == 0))
            proc.setStatusFlag('C', shifter_carry_out)

        print "Ejecutaste un orr"

class ARMMovInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001101111000000000000000000000
    bitmask  = 0b00000001101000000000000000000000

    def execute(self, proc, inst):
        shifter_operand   = 0
        shifter_carry_out = 0
        if not checkCondition(proc, inst):
            return

        S  = utils.getBits(inst, 20)
        I  = utils.getBits(inst, 25)
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)

        modes =  component.getAllUtilitiesRegisteredFor(IARMDPAddressingMode)
        for mode in modes:
            if testInstruction(mode, inst):
                shifter_operand, shifter_carry_out = mode.getVal(proc, inst)
                break
                
        setattr(proc, rd, shifter_operand)

        if S == 1 and rd == 'r15':
            try:
                proc.cpsr = spsr
            except:
                print "UNPREDICTABLE"
        elif S == 1:
            proc.setStatusFlag('N', utils.getBits(shifter_operand, 31))
            proc.setStatusFlag('Z', int(shifter_operand == 0))
            proc.setStatusFlag('C', shifter_carry_out)

class ARMBicInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001101111000000000000000000000
    bitmask  = 0b00000001110000000000000000000000

    def execute(self, proc, inst):
        shifter_operand   = 0
        shifter_carry_out = 0
        if not checkCondition(proc, inst):
            return

        S  = utils.getBits(inst, 20)
        I  = utils.getBits(inst, 25)
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)
        rn = utils.getBits(inst, 16, 4)
        rn = 'r' + str(rn)

        modes =  component.getAllUtilitiesRegisteredFor(IARMDPAddressingMode)
        for mode in modes:
            if testInstruction(mode, inst):
                shifter_operand, shifter_carry_out = mode.getVal(proc, inst)
                break

        shifterBitset = Bitset(shifter_operand, 32)
        shifter_operand = int(~shifterBitset)
	
        setattr(proc, rd, getattr(proc, rn) & shifter_operand)
        if S == 1 and rd == 'r15':
            try:
                proc.cpsr = proc.spsr
            except:
                print "UNPREDICTABLE"
        elif S == 1:
            proc.setStatusFlag('N', utils.getBits(getattr(proc, rd), 31))
            proc.setStatusFlag('Z', int(getattr(proc, rd) == 0))
            proc.setStatusFlag('C', shifter_carry_out)

class ARMMvnInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001101111000000000000000000000
    bitmask  = 0b00000001111000000000000000000000

    def execute(self, proc, inst):
        shifter_operand   = 0
        shifter_carry_out = 0
        if not checkCondition(proc, inst):
            return

        S  = utils.getBits(inst, 20)
        I  = utils.getBits(inst, 25)
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)

        modes =  component.getAllUtilitiesRegisteredFor(IARMDPAddressingMode)
        for mode in modes:
            if testInstruction(mode, inst):
                shifter_operand, shifter_carry_out = mode.getVal(proc, inst)
                break

        shifterBitset = Bitset(shifter_operand, 32)
        shifter_operand = int(~shifterBitset)
                
        setattr(proc, rd, shifter_operand)

        if S == 1 and rd == 'r15':
            try:
                proc.cpsr = spsr
            except:
                print "UNPREDICTABLE"
        elif S == 1:
            proc.setStatusFlag('N', utils.getBits(shifter_operand, 31))
            proc.setStatusFlag('Z', int(shifter_operand == 0))
            proc.setStatusFlag('C', shifter_carry_out)


class ARMDPAddressingMode1(object):
    implements(IARMDPAddressingMode)

    testmask = 0b00001110000000000000000000000000
    bitmask  = 0b00000010000000000000000000000000

    def getVal(self, proc, inst):
        rotate_imm = utils.getBits(inst, 8, 4)
        immed_8    = utils.getBits(inst, 0, 8)
        shifter_operand = utils.ror(immed_8, 32, rotate_imm * 2)
        if rotate_imm == 0:
            shifter_carry_out = proc.statusFlag('C')
        else:
            shifter_carry_out = utils.getBits(shifter_operand, 31)
        return (shifter_operand, shifter_carry_out)

class ARMDPAddressingMode2(object):
    implements(IARMDPAddressingMode)

    testmask = 0b00001110000000000000111111110000
    bitmask  = 0b00000000000000000000000000000000

    def getVal(self, proc, inst):
        rm = utils.getBits(inst, 0, 4)
        rm = 'r' + str(rm)
        shifter_operand = getattr(proc, rm)
        shifter_carry_out = proc.statusFlag('C')
        return (shifter_operand, shifter_carry_out)

class ARMDPAddressingMode3(object):
    implements(IARMDPAddressingMode)

    testmask = 0b00001110000000000000000001110000
    bitmask  = 0b00000000000000000000000000000000

    def getVal(self, proc, inst):
        rm = utils.getBits(inst, 0, 4)
        rm = 'r' + str(rm)
        shift_imm = utils.getBits(inst, 7, 5)
        if shift_imm == 0:
            shifter_operand = getattr(proc, rm)
            shifter_carry_out = proc.statusFlag('C')
        else:
            shifter_operand = getattr(proc, rm) << shift_imm
            shifter_operand = shifter_operand & 0xFFFFFFFF
            shifter_carry_out = utils.getBits(getattr(proc, rm), 32 - shift_imm)
        
        return (shifter_operand, shifter_carry_out)

class ARMDPAddressingMode4(object):
    implements(IARMDPAddressingMode)

    testmask = 0b00001110000000000000000011110000
    bitmask  = 0b00000000000000000000000000010000

    def getVal(self, proc, inst):
        rm = utils.getBits(inst, 0, 4)
        rm = 'r' + str(rm)
        rs = utils.getBits(inst, 8, 4)
        rs = 'r' + str(rs)
        rs_val = utils.getBits(getattr(proc, rs), 0, 8)
        if rs_val == 0:
            shifter_operand = getattr(proc, rm)
            shifter_carry_out = proc.statusFlag('C')
        elif rs_val < 32:
            shifter_operand = getattr(proc, rm) << rs_val
            shifter_operand = shifter_operand & 0xFFFFFFFF
            shifter_carry_out = utils.getBits(getattr(proc, rm), 32 - rs_val)
        elif rs_val == 32:
            shifter_operand = 0
            shifter_carry_out = utils.getBits(getattr(proc, rm), 0)
        else:
            shifter_operand = 0
            shifter_carry_out = 0 
            
        return (shifter_operand, shifter_carry_out)
        
        

class ARMDPAddressingMode5(object):
    implements(IARMDPAddressingMode)

    testmask = 0b00001110000000000000000001110000
    bitmask  = 0b00000000000000000000000000100000

    def getVal(self, proc, inst):
        rm = utils.getBits(inst, 0, 4)
        rm = 'r' + str(rm)
        shift_imm = utils.getBits(inst, 7, 5)
        if shift_imm == 0:
            shifter_operand = getattr(proc, rm)
            shifter_carry_out = utils.getBits(getattr(proc, rm), 31)
        else:
            shifter_operand = getattr(proc, rm) >> shift_imm
            shifter_carry_out = utils.getBits(getattr(proc, rm), shift_imm - 1)
        
        return (shifter_operand, shifter_carry_out)

class ARMDPAddressingMode6(object):
    implements(IARMDPAddressingMode)

    testmask = 0b00001110000000000000000011110000
    bitmask  = 0b00000000000000000000000000110000

    def getVal(self, proc, inst):
        rm = utils.getBits(inst, 0, 4)
        rm = 'r' + str(rm)
        rs = utils.getBits(inst, 8, 4)
        rs = 'r' + str(rs)
        rs_val = utils.getBits(getattr(proc, rs), 0, 8)
        if rs_val == 0:
            shifter_operand = getattr(proc, rm)
            shifter_carry_out = proc.statusFlag('C')
        elif rs_val < 32:
            shifter_operand = getattr(proc, rm) >> rs_val
            shifter_carry_out = utils.getBits(getattr(proc, rm), rs_val - 1)
        elif rs_val == 32:
            shifter_operand = 0
            shifter_carry_out = utils.getBits(getattr(proc, rm), 1)
        else:
            shifter_operand = 0
            shifter_carry_out = 0 
            
        return (shifter_operand, shifter_carry_out)

class ARMDPAddressingMode7(object):
    implements(IARMDPAddressingMode)

    testmask = 0b00001110000000000000000001110000
    bitmask  = 0b00000000000000000000000001000000

    def getVal(self, proc, inst):
        rm = utils.getBits(inst, 0, 4)
        rm = 'r' + str(rm)
        rmVal = getattr(proc, rm)
        shift_imm = utils.getBits(inst, 7, 5)
        if shift_imm == 0:
            if utils.getBits(rmVal, 31) == 0:
                shifter_operand = 0
                shifter_carry_out = utils.getBits(rmVal, 31)
            else:
                shifter_operand = 0xFFFFFFFF
                shifter_carry_out = utils.getBits(rmVal, 31)
        else:
            shifter_operand = utils.asr(rmVal, shift_imm)
            shifter_carry_out = utils.getBits(rmVal, shift_imm - 1)
    
        return (shifter_operand, shifter_carry_out)

class ARMDPAddressingMode8(object):
    implements(IARMDPAddressingMode)

    testmask = 0b00001110000000000000000011110000
    bitmask  = 0b00000000000000000000000001010000

    def getVal(self, proc, inst):
        rm = utils.getBits(inst, 0, 4)
        rm = 'r' + str(rm)
        rs = utils.getBits(inst, 8, 4)
        rs = 'r' + str(rs)
        rs_val = utils.getBits(getattr(proc, rs), 0, 8)
        if rs_val == 0:
            shifter_operand = getattr(proc, rm)
            shifter_carry_out = proc.statusFlag('C')
        elif rs_val < 32:
            shifter_operand = utils.asr(getattr(proc, rm), rs_val)
            shifter_carry_out = utils.getBits(getattr(proc, rm), rs_val - 1)
        else:
            if utils.getBits(getattr(proc, rm), 31) == 0:
                shifter_operand = 0
                shifter_carry_out = utils.getBits(getattr(proc, rm), 31)
            else:
                shifter_operand = 0xFFFFFFFF
                shifter_carry_out = utils.getBits(getattr(proc, rm), 31)

        return (shifter_operand, shifter_carry_out)

class ARMDPAddressingMode9(object):
    implements(IARMDPAddressingMode)

    testmask = 0b00001110000000000000000001110000
    bitmask  = 0b00000000000000000000000001100000

    def getVal(self, proc, inst):
        rm = utils.getBits(inst, 0, 4)
        rm = 'r' + str(rm)
        rmVal = getattr(proc, rm)
        shift_imm = utils.getBits(inst, 7, 5)
        if shift_imm == 0:
            operand1 = proc.statusFlag('C') << 31
            operand1 = operand1 & 0xFFFFFFFF
            operand2 = rmVal >> 1
            shifter_operand = operand1 | operand2
            shifter_carry_out = utils.getBits(rmVal, 0)
        else:
            shifter_operand = utils.ror(rmVal, 32, shift_imm)
            shifter_carry_out = utils.getBits(rmVal, shift_imm - 1)

        return (shifter_operand, shifter_carry_out)

class ARMDPAddressingMode10(object):
    implements(IARMDPAddressingMode)

    testmask = 0b00001110000000000000000011110000
    bitmask  = 0b00000000000000000000000001110000

    def getVal(self, proc, inst):
        rm = utils.getBits(inst, 0, 4)
        rm = 'r' + str(rm)
        rs = utils.getBits(inst, 8, 4)
        rs = 'r' + str(rs)
        rs_val = utils.getBits(getattr(proc, rs), 0, 8)
        rs_val2 = utils.getBits(getattr(proc, rs), 0, 5)
        if rs_val == 0:
            shifter_operand = getattr(proc, rm)
            shifter_carry_out = proc.statusFlag('C')
        elif rs_val2 == 0:
            shifter_operand = getattr(proc, rm)
            shifter_carry_out = utils.getBits(getattr(proc, rm), 31)
        else:
            shifter_operand = utils.ror(getattr(proc, rm), 32, rs_val2)
            shifter_carry_out = utils.getBits(getattr(proc, rm), rs_val2 - 1)

        return (shifter_operand, shifter_carry_out)




"""
This is the main simulator module, here the main function is 
contained
"""
#Copyright 2009 armsim authors.
#
#This file is part of armsim.
#
#armsim is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#armsim is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with armsim.  If not, see <http://www.gnu.org/licenses/>.

from armsim.interfaces.armprocessor import IARMProcessor
from armsim.interfaces.arminstruction import IARMInstruction
from armsim.utils import testInstruction
from threading import Thread
from armsim import utils
from zope import component 
from optparse import OptionParser
from code import interact
import sys

class ARMSim(Thread):

    def __init__(self):
        Thread.__init__(self)

    def run(self):
        processor = component.queryUtility(IARMProcessor)
        processor.run()

def init(program, options):
    processor = component.queryUtility(IARMProcessor)
    processor.initMemory(int(options.memory_size))
 
    if options.halt:
        processor.halt()

    i = int(options.load_addr)

    while(True):
        c = program.read(1)
        if c == '':
            break
        processor.memory[i] = ord(c)
        i += 1

    processor.pc = int(options.entry_point)

    program.close()

    run()


def run():
    sim = ARMSim()
    processor = component.queryUtility(IARMProcessor)
    ic_locals = {}
    ic_locals['processor'] = processor
    ic_locals['copyright'] = "Copyright (C) 2009 armsim authors, check the authors in the source AUTHORS file"
    sim.start()
    banner = """armsim Copyright (C) 2009 armsim authors
This program comes with ABSOLUTELY NO WARRANTY.
This program is free software, and you are welcome to redistribute
it under certain conditions, check GPLv3 text for details
             """
    interact(banner, local=ic_locals)



def main():
    usage  = "usage: %prog [options] arm_program"
    parser = OptionParser(usage=usage)
    parser.add_option('-e', '--entry', dest='entry_point', default=0,
                      help='Entry point where te program starts, default 0')
    parser.add_option('-l', '--loadaddr', dest='load_addr', default=0,
                      help='Load addr of the program, default 0')
    parser.add_option('-m', '--memsize', dest='memory_size', default=128*1024,
                      help='Size of memory in bytes, default 131072 (128k)')
    parser.add_option('-H', '--halt', dest='halt', 
                      default=False, action='store_true',
                      help='Dont run the program automatically')
    (options, args) = parser.parse_args()

    if len(args) != 1:
        parser.print_help()
        return
    else:
        try:
            programName = args[0]
            program = open(programName, 'r')
        except IOError:
            print "ARM Program %s does not exist" % programName
            return
    
    init(program, options)




"""
"""
#Copyright 2009 armsim authors.
#
#This file is part of armsim.
#
#armsim is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#armsim is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with armsim.  If not, see <http://www.gnu.org/licenses/>.


from armsim.interfaces.arminstruction import IARMInstruction
from armsim import utils
from armsim.bitset import Bitset
from zope.interface import implements

class ARMBlInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001110000000000000000000000000
    bitmask  = 0b00001010000000000000000000000000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return

        l = utils.getBits(inst, 24)
        signed_immed_24 = utils.getBits(inst, 0, 24)

        if l == 1:
            proc.lr = proc.pc

        branch_addr = utils.signExtend(signed_immed_24, 24, 30)
        branch_addr = branch_addr << 2
        branch_addr = utils.signValue(branch_addr)
        proc.pc += 4 + branch_addr


class ARMBlx1Instruction(object):
    implements(IARMInstruction)

    testmask = 0b11111110000000000000000000000000
    bitmask  = 0b11111010000000000000000000000000

    def execute(self, proc, inst):
        raise Exception("Thumb instruction set not implemented")

class ARMBlx2Instruction(object):
    implements(IARMInstruction)

    testmask = 0b00001111111100000000000011110000
    bitmask  = 0b00000001001000000000000000110000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return
        target = utils.getBits(inst, 0, 4)
        proc.lr = proc.pc
        t = utils.getBits(target, 0)
        if t:
            raise Exception("Thumb instruction set not implemented")
        proc.pc = target & 0xFFFFFFFE
            

class ARMBxInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001111111100000000000011110000
    bitmask  = 0b00000001001000000000000000010000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return
        target = utils.getBits(inst, 0, 4)
        t = utils.getBits(target, 0)
        if t:
            raise Exception("Thumb instruction set not implemented")
        proc.pc = target & 0xFFFFFFFE




from setuptools import setup, find_packages, Extension
import sys, os

version = '0.1'

setup(name='armsim',
      version=version,
      description="ARM Simulator for Linux",
      long_description="""\
armsim is an ARM Simulator for Linux

Objective
---------
armsim has the objective of simulating arm based processors starting with the
at91sam9260

nonetheless, it is designed to be extensible, so you'll be able to write you
own classes to plug in you microcontroller

Is is designed for Linux and Python2.6 so be sure you have these requirements.

Current Status
--------------
Currently a total of 20 instructions are supported, even tough most of them
are recognized.

The project is being developed quickly so you can expect the complete
instruction set to be supported in a few weeks

                       """,
      classifiers=[
          'Development Status :: 1 - Planning',
          'Operating System :: POSIX :: Linux',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: C',
          'Topic :: Software Development :: Embedded Systems',
          'Topic :: System :: Emulators',
      ], 
      author='Eduardo Diaz',
      author_email='iamedu@gmail.com',
      url='http://code.google.com/p/armdev/',
      license='GPL',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'zope.component',
      ],
      entry_points={
          'console_scripts':[
              'armsim = armsim.simulator:main',
          ]
      },
      ext_modules=[
          Extension('uart', ['src/uartmodule.c'], libraries=['util'])
      ],
      platforms=['Linux'],
      package_dir={'armsim' : 'armsim'},
      package_data={'armsim' : ['controllers/*.cfg']},
      )




"""
Load and store instructions
---------------------------
"""
#Copyright 2009 armsim authors.
#
#This file is part of armsim.
#
#armsim is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#armsim is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with armsim.  If not, see <http://www.gnu.org/licenses/>.

from armsim.interfaces.arminstruction import IARMInstruction
from armsim.interfaces.arminstruction import IARMLSAddressingMode
from armsim.interfaces.arminstruction import IARMLSMAddressingMode
from armsim import exceptions
from zope.interface import implements
from zope import component
from armsim import utils
from armsim.bitset import Bitset

class ARMLdm1Instruction(object):
    implements(IARMInstruction)

    testmask = 0b00001110010100000000000000000000
    bitmask  = 0b00001000000100000000000000000000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return

        modes =  component.getAllUtilitiesRegisteredFor(IARMLSMAddressingMode)
        for mode in modes:
            if utils.testInstruction(mode, inst):
                start_address, end_address = mode.getVal(proc, inst)
                break

        register_list = Bitset(utils.getBits(inst, 0, 16), 32)

        address = start_address
  
        for i in range(15):
            if register_list[i]:
                setattr(proc, 'r' + str(i), proc.readAddr(address, 4))
                address += 4

        if register_list[15]:
            value = proc.readAddr(address, 4)
            proc.pc = value & 0xFFFFFFFE
            address += 4

        if end_address != address - 4:
            raise exceptions.DataAbort()

class ARMLdm2Instruction(object):
    implements(IARMInstruction)

    testmask = 0b00001110011100001000000000000000
    bitmask  = 0b00001000010100000000000000000000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return

        modes =  component.getAllUtilitiesRegisteredFor(IARMLSMAddressingMode)
        for mode in modes:
            if utils.testInstruction(mode, inst):
                start_address, end_address = mode.getVal(proc, inst)
                break
        print "Ejecutaste un ldm(2)"

class ARMLdm3Instruction(object):
    implements(IARMInstruction)

    testmask = 0b00001110010100001000000000000000
    bitmask  = 0b00001000010100001000000000000000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return

        modes =  component.getAllUtilitiesRegisteredFor(IARMLSMAddressingMode)
        for mode in modes:
            if utils.testInstruction(mode, inst):
                start_address, end_address = mode.getVal(proc, inst)
                break
        print "Ejecutaste un ldm(3)"

class ARMLdrInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001100010100000000000000000000
    bitmask  = 0b00000100000100000000000000000000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return

        modes =  component.getAllUtilitiesRegisteredFor(IARMLSAddressingMode)
        for mode in modes:
            if utils.testInstruction(mode, inst):
                address = mode.getVal(proc, inst)
                break
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)

        data = proc.readAddr(address)

        if rd == 'r15':
            proc.pc = data & 0xfffffffe
        else:
            setattr(proc, rd, data)

class ARMLdrbInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001100010100000000000000000000
    bitmask  = 0b00000100010100000000000000000000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return

        modes =  component.getAllUtilitiesRegisteredFor(IARMLSAddressingMode)
        for mode in modes:
            if utils.testInstruction(mode, inst):
                address = mode.getVal(proc, inst)
                break
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)

        data = proc.readAddr(address, 1)

        setattr(proc, rd, data)

class ARMLdrbtInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001101011100000000000000000000
    bitmask  = 0b00000100011100000000000000000000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return

        modes =  component.getAllUtilitiesRegisteredFor(IARMLSAddressingMode)
        for mode in modes:
            if utils.testInstruction(mode, inst):
                address = mode.getVal(proc, inst)
                break
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)
        rn = utils.getBits(inst, 16, 4)
        rn = 'r' + str(rn)

        data = proc.readAddr(address, 1)
        setattr(proc, rd, data)
        setattr(proc, rn, address)

class ARMLdrdInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001110000100000000000011110000
    bitmask  = 0b00000000000000000000000011010000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return

        modes =  component.getAllUtilitiesRegisteredFor(IARMLSAddressingMode)
        for mode in modes:
            if utils.testInstruction(mode, inst):
                address = mode.getVal(proc, inst)
                break
        rd = utils.getBits(inst, 12, 4)
        rn = rd + 1
        rd = 'r' + str(rd)
        rn = 'r' + str(rn)

        data = proc.readAddr(address)
        setattr(proc, rd, data)
        data = proc.readAddr(address+4)
        setattr(proc, rn, data)

class ARMLdrhInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001110000100000000000011110000
    bitmask  = 0b00000000000100000000000010110000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return

        modes =  component.getAllUtilitiesRegisteredFor(IARMLSAddressingMode)
        for mode in modes:
            if utils.testInstruction(mode, inst):
                address = mode.getVal(proc, inst)
                break
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)

        data = proc.readAddr(address, 2)
        setattr(proc, rd, data)

class ARMLdrsbInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001110000100000000000011110000
    bitmask  = 0b00000000000100000000000011010000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return

        modes =  component.getAllUtilitiesRegisteredFor(IARMLSAddressingMode)
        for mode in modes:
            if utils.testInstruction(mode, inst):
                address = mode.getVal(proc, inst)
                break
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)

        data = proc.readAddr(address, 1)
        data = utils.signExtend(data, 8, 32)

        setattr(proc, rd, data)

class ARMLdrshInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001110000100000000000011110000
    bitmask  = 0b00000000000100000000000011110000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return

        modes =  component.getAllUtilitiesRegisteredFor(IARMLSAddressingMode)
        for mode in modes:
            if utils.testInstruction(mode, inst):
                address = mode.getVal(proc, inst)
                break
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)

        data = proc.readAddr(address, 2)
        data = utils.signExtend(data, 16, 32)

        setattr(proc, rd, data)

class ARMLdrtInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001101011100000000000000000000
    bitmask  = 0b00000100001100000000000000000000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return

        modes =  component.getAllUtilitiesRegisteredFor(IARMLSAddressingMode)
        for mode in modes:
            if utils.testInstruction(mode, inst):
                address = mode.getVal(proc, inst)
                break
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)

        data = proc.readAddr(address)

        setattr(proc, rd, data)

class ARMStm1Instruction(object):
    implements(IARMInstruction)

    testmask = 0b00001110010100000000000000000000
    bitmask  = 0b00001000000000000000000000000000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return

        modes =  component.getAllUtilitiesRegisteredFor(IARMLSMAddressingMode)
        for mode in modes:
            if utils.testInstruction(mode, inst):
                start_address, end_address = mode.getVal(proc, inst)
                break

        register_list = Bitset(utils.getBits(inst, 0, 16), 32)

        address = start_address
  
        for i in range(16):
            if register_list[i]:
                proc.saveAddr(address, getattr(proc, 'r' + str(i)))
                address += 4

        if end_address != address - 4:
            raise exceptions.DataAbort()

class ARMStm2Instruction(object):
    implements(IARMInstruction)

    testmask = 0b00001110011100000000000000000000
    bitmask  = 0b00001000010000000000000000000000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return

        modes =  component.getAllUtilitiesRegisteredFor(IARMLSMAddressingMode)
        for mode in modes:
            if utils.testInstruction(mode, inst):
                start_address, end_address = mode.getVal(proc, inst)
                break
        print "Ejecutaste un stm(2)"

class ARMStrInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001100010100000000000000000000
    bitmask  = 0b00000100000000000000000000000000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return

        modes =  component.getAllUtilitiesRegisteredFor(IARMLSAddressingMode)
        for mode in modes:
            if utils.testInstruction(mode, inst):
                address = mode.getVal(proc, inst)
                break
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)

        proc.saveAddr(address, getattr(proc, rd))


class ARMStrbInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001100010100000000000000000000
    bitmask  = 0b00000100010000000000000000000000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return

        modes =  component.getAllUtilitiesRegisteredFor(IARMLSAddressingMode)
        for mode in modes:
            if utils.testInstruction(mode, inst):
                address = mode.getVal(proc, inst)
                break
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)

        value = getattr(proc, rd)
        value = utils.getBits(value, 0, 8)

        proc.saveAddr(address, value, 1)

class ARMStrbtInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001101011100000000000000000000
    bitmask  = 0b00000100011000000000000000000000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return

        modes =  component.getAllUtilitiesRegisteredFor(IARMLSAddressingMode)
        for mode in modes:
            if utils.testInstruction(mode, inst):
                address = mode.getVal(proc, inst)
                break
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)

        value = getattr(proc, rd)
        value = utils.getBits(value, 0, 8)

        proc.saveAddr(address, value, 1)

class ARMStrdInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001110000100000000000011110000
    bitmask  = 0b00000000000000000000000011110000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return

        modes =  component.getAllUtilitiesRegisteredFor(IARMLSAddressingMode)
        for mode in modes:
            if utils.testInstruction(mode, inst):
                address = mode.getVal(proc, inst)
                break
        rd = utils.getBits(inst, 12, 4)
        rn = rd + 1
        rd = 'r' + str(rd)
        rn = 'r' + str(rn)

        value = getattr(proc, rd)
        proc.saveAddr(address, value)

        value = getattr(proc, rn)
        proc.saveAddr(address + 4, value)

class ARMStrhInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001110000100000000000011110000
    bitmask  = 0b00000000000000000000000010110000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return

        modes =  component.getAllUtilitiesRegisteredFor(IARMLSAddressingMode)
        for mode in modes:
            if utils.testInstruction(mode, inst):
                address = mode.getVal(proc, inst)
                break
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)

        value = getattr(proc, rd)
        value = utils.getBits(value, 0, 16)

        proc.saveAddr(address, value, 2)

class ARMStrtInstruction(object):
    implements(IARMInstruction)

    testmask = 0b00001101011100000000000000000000
    bitmask  = 0b00000100001000000000000000000000

    def execute(self, proc, inst):
        if not utils.checkCondition(proc, inst):
            return

        modes =  component.getAllUtilitiesRegisteredFor(IARMLSAddressingMode)
        for mode in modes:
            if utils.testInstruction(mode, inst):
                address = mode.getVal(proc, inst)
                break
        rd = utils.getBits(inst, 12, 4)
        rd = 'r' + str(rd)

        proc.saveAddr(address, getattr(proc, rd))

#Addressing Mode 2 - Load and Store Word or Unsigned Byte
class ARMLSAddressingMode1(object):
    implements(IARMLSAddressingMode)

    testmask = 0b00001111001000000000000000000000
    bitmask  = 0b00000101000000000000000000000000

    def getVal(self, proc, inst):
        u         = utils.getBits(inst, 23)
        rn        = utils.getBits(inst, 16, 4)
        rn        = 'r' + str(rn)
        rnVal     = getattr(proc, rn)
        offset_12 = utils.getBits(inst, 0, 12)
        if u == 1:
            address = rnVal + offset_12
        else:
            address = rnVal - offset_12
        return address


class ARMLSAddressingMode2(object):
    implements(IARMLSAddressingMode)

    testmask = 0b00001111001000000000111111110000
    bitmask  = 0b00000111000000000000000000000000

    def getVal(self, proc, inst):
        u         = utils.getBits(inst, 23)
        rn        = utils.getBits(inst, 16, 4)
        rn        = 'r' + str(rn)
        rnVal     = getattr(proc, rn)
        rm        = utils.getBits(inst, 0, 4)
        rm        = 'r' + str(rm)
        rmVal     = getattr(proc, rm)
        if u == 1:
            address = rnVal + rmVal
        else:
            address = rnVal - rmVal
        return address

class ARMLSAddressingMode3(object):
    implements(IARMLSAddressingMode)

    testmask = 0b00001111001000000000000000010000
    bitmask  = 0b00000111000000000000000000000000

    def getVal(self, proc, inst):
        u         = utils.getBits(inst, 23)
        rn        = utils.getBits(inst, 16, 4)
        rn        = 'r' + str(rn)
        rnVal     = getattr(proc, rn)
        rm        = utils.getBits(inst, 0, 4)
        rm        = 'r' + str(rm)
        rmVal     = getattr(proc, rm)

        shift     = utils.getBits(inst, 5, 2)
        shift_imm = utils.getBits(inst, 7, 5)

        if shift == 0: #LSL
            index = rmVal << shift_imm
            index &= 0xFFFFFFFF
        elif shift == 1: #LSR
            if shift_imm == 0:
                index = 0
            else:
                index = rmVal >> shift_imm
        elif shift == 2: #ASR
            if shift_imm == 0:
                if utils.getBits(rmVal, 31) == 1:
                    index = 0xFFFFFFFF
                else:
                    index = 0
            else:
                index = utils.asr(rmVal, shift_imm)
        elif shift == 3: #ROR or RRX
            if shift_imm == 0:
                index = (proc.statusFlag('C') << 31) | (rmVal >> 1)
            else:
                index = utils.ror(rmVal, 32, shift_imm)

        if u == 1:
            address = rnVal + index
        else:
            address = rnVal - index
        return address

class ARMLSAddressingMode4(object):
    implements(IARMLSAddressingMode)

    testmask = 0b00001111001000000000000000000000
    bitmask  = 0b00000101001000000000000000000000

    def getVal(self, proc, inst):
        u         = utils.getBits(inst, 23)
        rn        = utils.getBits(inst, 16, 4)
        rn        = 'r' + str(rn)
        rnVal     = getattr(proc, rn)
        offset_12 = utils.getBits(inst, 0, 12)
        if u == 1:
            address = rnVal + offset_12
        else:
            address = rnVal - offset_12

        if utils.checkCondition(proc, inst):
            setattr(proc, rn, address)

        return address

class ARMLSAddressingMode5(object):
    implements(IARMLSAddressingMode)

    testmask = 0b00001111001000000000111111110000
    bitmask  = 0b00000111001000000000000000000000

    def getVal(self, proc, inst):
        u         = utils.getBits(inst, 23)
        rn        = utils.getBits(inst, 16, 4)
        rn        = 'r' + str(rn)
        rnVal     = getattr(proc, rn)
        rm        = utils.getBits(inst, 0, 4)
        rm        = 'r' + str(rm)
        rmVal     = getattr(proc, rm)

        if u == 1:
            address = rnVal + rmVal
        else:
            address = rnVal - rmVal

        if utils.checkCondition(proc, inst):
            setattr(proc, rn, address)

        return address

class ARMLSAddressingMode6(object):
    implements(IARMLSAddressingMode)

    testmask = 0b00001111001000000000000000010000
    bitmask  = 0b00000111001000000000000000000000

    def getVal(self, proc, inst):
        u         = utils.getBits(inst, 23)
        rn        = utils.getBits(inst, 16, 4)
        rn        = 'r' + str(rn)
        rnVal     = getattr(proc, rn)
        rm        = utils.getBits(inst, 0, 4)
        rm        = 'r' + str(rm)
        rmVal     = getattr(proc, rm)

        shift     = utils.getBits(inst, 5, 2)
        shift_imm = utils.getBits(inst, 7, 5)

        if shift == 0: #LSL
            index = rmVal << shift_imm
            index &= 0xFFFFFFFF
        elif shift == 1: #LSR
            if shift_imm == 0:
                index = 0
            else:
                index = rmVal >> shift_imm
        elif shift == 2: #ASR
            if shift_imm == 0:
                if utils.getBits(rmVal, 31) == 1:
                    index = 0xFFFFFFFF
                else:
                    index = 0
            else:
                index = utils.asr(rmVal, shift_imm)
        elif shift == 3: #ROR or RRX
            if shift_imm == 0:
                index = (proc.statusFlag('C') << 31) | (rmVal >> 1)
            else:
                index = utils.ror(rmVal, 32, shift_imm)

        if u == 1:
            address = rnVal + index
        else:
            address = rnVal - index

        if utils.checkCondition(proc, inst):
            setattr(proc, rn, address)

        return address

#Load and Store Multiple Addressing modes
class ARMLSMAddressingMode1(object):
    implements(IARMLSMAddressingMode)

    testmask = 0b00001111100000000000000000010000
    bitmask  = 0b00001000100000000000000000000000

    def getVal(self, proc, inst):
        rn = utils.getBits(inst, 16, 4)
        rn = 'r' + str(rn)
        register_list = utils.getBits(inst, 0, 16)
        rnVal = getattr(proc, rn)
        
        start_address = rnVal
        end_address = rnVal + (utils.countSetBits(register_list) * 4) - 4

        if utils.checkCondition(proc, inst):
            setattr(proc, rn, end_address + 4)

        return (start_address, end_address)

class ARMLSMAddressingMode2(object):
    implements(IARMLSMAddressingMode)

    testmask = 0b00001111100000000000000000010000
    bitmask  = 0b00001001100000000000000000000000

    def getVal(self, proc, inst):
        rn = utils.getBits(inst, 16, 4)
        rn = 'r' + str(rn)
        register_list = utils.getBits(inst, 0, 16)
        rnVal = getattr(proc, rn)
        w = utils.getBits(inst, 21)
        
        start_address = rnVal + 4
        end_address = rnVal + (utils.countSetBits(register_list) * 4)

        if utils.checkCondition(proc, inst) and w == 1:
            setattr(proc, rn, end_address)

        return (start_address, end_address)

class ARMLSMAddressingMode3(object):
    implements(IARMLSMAddressingMode)

    testmask = 0b00001111100000000000000000010000
    bitmask  = 0b00001000000000000000000000000000

    def getVal(self, proc, inst):
        rn = utils.getBits(inst, 16, 4)
        rn = 'r' + str(rn)
        register_list = utils.getBits(inst, 0, 16)
        rnVal = getattr(proc, rn)
        w = utils.getBits(inst, 21)
        
        start_address = rnVal - (utils.countSetBits(register_list) * 4) + 4
        end_address = rnVal

        if utils.checkCondition(proc, inst) and w == 1:
            setattr(proc, rn, start_address - 4)

        return (start_address, end_address)

class ARMLSMAddressingMode4(object):
    implements(IARMLSMAddressingMode)

    testmask = 0b00001111100000000000000000010000
    bitmask  = 0b00001001000000000000000000000000

    def getVal(self, proc, inst):
        rn = utils.getBits(inst, 16, 4)
        rn = 'r' + str(rn)
        register_list = utils.getBits(inst, 0, 16)
        rnVal = getattr(proc, rn)
        w = utils.getBits(inst, 21)
        
        start_address = rnVal - (utils.countSetBits(register_list) * 4)
        end_address = rnVal - 4

        if utils.checkCondition(proc, inst) and w == 1:
            setattr(proc, rn, start_address)

        return (start_address, end_address)

